from charmhelpers.core.hookenv import (
    log,
    relation_id,
    relation_set,
    unit_get,
)

from charmhelpers.contrib.hahelpers.cluster import (
    oldest_peer,
    peer_units,
    peer_ips,
)

REPCACHED_REPLICA_FILE = "/var/run/repcached_replica"
REPCACHED_MASTER_WAIT = 10


def store_replica(master, replica):
    with open(REPCACHED_REPLICA_FILE, 'w+') as fd:
        fd.write("%s,%s" % (master, replica))


def get_current_replica():
    replica = get_repcached_replica()

    if replica:
        master, secondary = replica
        ip_addr = unit_get('private-address')
        if ip_addr == master:
            replica = secondary
        elif ip_addr == secondary:
            replica = master
    else:
        peers = peer_units()

        if len(peers) > 1:
            msg = """Cannot configure replication between more than 2 units,
            all the units are going to be setup as standalone. Please
            remove the remaining units with 'juju remove-unit', and
            then set 'juju set memcached repcached=true' again"""
            log(msg, level='WARN')
        else:
            try:
                peer_unit = peer_ips().values()[0]

                if oldest_peer(peers):
                    master, secondary = (unit_get('private-address'),
                                         peer_unit)
                    replica = secondary
                else:
                    master, secondary = (peer_unit,
                                         unit_get('private-address'))
                    replica = master

                store_replica(master, secondary)

            except IndexError:
                replica = None

    log("Current replica value: %s" % replica, level="INFO")
    return replica


def get_repcached_replica():
    try:
        with open(REPCACHED_REPLICA_FILE, 'r') as fd:
            return tuple(fd.read().split(','))
    except:
        return None


def set_repcached_replica():
    replica = peer_ips().values()[0]
    log("Setting replica unit: %s" % replica)

    master = unit_get('private-address')
    relation_set(relation_id(), {
        'master': master,
        'replica': replica,
    })

    with open(REPCACHED_REPLICA_FILE, 'w+') as fd:
        fd.write("%s,%s" % (master, replica))
    return (master, replica)
